from zipdata import data, zip2state
import math, sys, os, gzip
from config import distance_ranges, datadir, same_state
distance_ranges.sort() # make sure ranges are in ascending order

class coord:
    def __init__(self, zip, lat, long):
        self.zip = zip
        self.latitude = float(lat)
        self.longitude = float(long)

    def __str__(self):
        return """Latitude: %s\nLongitude: %s\n""" % (self.latitude, self.longitude)

coords = {}
for d in data.split('\n'):
    d = d.strip()
    if not d: continue

    zip, lat, long = d.split(' ')
    coords[zip] = coord(zip, lat, long)


from math import radians, fabs, sqrt, sin, pow, cos, atan2

def distance(A, B):
    lat1 = radians(A.latitude)
    lat2 = radians(B.latitude)
    long1 = radians(A.longitude)
    long2 = radians(B.longitude)
    dlat = lat2 - lat1
    dlong = long2 - long1

    l = (lat1 + lat2) / 2
    a = 6378
    b = 6357
    ravg = (a +b)/2 # a simple average value for the radius of the earth

    a = (1-cos(dlat))/2 + cos(lat1) * cos(lat2) * (1-cos(dlong))/2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))
    d = ravg * c # distance in kilometers
    dmiles = d * 0.621371192
    return dmiles

if __name__ == '__main__':
    ofile = gzip.open(os.path.join(datadir,'data.gz'), 'wb')
    efile = open('zip_errors.log', 'wb')
    count = 0
    for zip, loc in coords.items():
        try:
            state = zip2state(zip)
        except KeyError:
            print >>efile, 'skipping %s not a state zipcode' % zip
            continue
        print zip
        print >> ofile, zip
        values = {}
        for range in distance_ranges:
            values[range] = []
        for zip2, loc2 in coords.items():
            try:
                if zip == zip2 or (same_state and state != zip2state(zip2)):
                    continue
            except KeyError:
                continue
            dist = int(distance(loc, loc2))
            if dist > max(distance_ranges):
                continue
            prev = 0
            for range in distance_ranges:
                if dist > prev and dist <= range:
                    values[range].append(zip2)
        for range in distance_ranges:
            print >> ofile, ', '.join(values[range])
        count = count + 1
        if not count % 500:
            ofile.flush()
            efile.flush()
    ofile.flush()
    efile.flush()
    ofile.close()
    efile.close()
    print 'fin'
